@@ -30,6 +30,18 @@ my %default_values = (
         PERMISSIONS      => 0755,
         SAVE_AS_DEFAULTS => 0,
         USAGE_MESSAGE    => $usage,
+        FIRST_TEST_NUMBER                   => 1,
+        TEST_NUMBER_FORMAT                  => "%03d",
+        TEST_NAME                           => 'load',
+        EXTRA_MODULES_SINGLE_TEST_FILE      => 0,
+        TEST_NAME_DERIVED_FROM_MODULE_NAME  => 0,
+        TEST_NAME_SEPARATOR                 => q{_},
+        INCLUDE_MANIFEST_SKIP               => 0,
+        INCLUDE_TODO                        => 1,
+        INCLUDE_POD_COVERAGE_TEST           => 0,
+        INCLUDE_POD_TEST                    => 0,
+        INCLUDE_LICENSE                     => 1,
+        INCLUDE_SCRIPTS_DIRECTORY           => 1,
 );
 
 sub default_values {
